#########################################################################################
###### Purpose: Script for Analyses in "Toward a novel laser-based approach for estimating snow interception"
###### Last updated: 11/4/2019
###### Authors: Micah Russell and Jan Eitel, University of Idaho
#########################################################################################


file_names = choose.files("INSERT ASCI II FILE LOCATION") ### Set this to a file path where your processed ATLS files are located

file_aoi = choose.files("INSERT ASCI II FILE LOCATION") ### Set file path to a ATLS point cloud that represents the area of interest (aoi) (i.e., tree)

tree_weight = read.table("INSERT CSV LOCATION", sep = ",", header = FALSE) ### read in file with tree weight data

study_year = "2018_"    ### Important - make sure to have a "_" when specifying the year (e.g., "2017_")

temps = read.table("INSERT CSV LOCATION", sep =",", header = TRUE)

### End user input
########################################################################################

require(rgl)            
require(alphashape3d)   
require(magic)          
require(rgl)
require(raster)
require(sp)
require(VoxR)
require(plot3D)
require(alphashape3d)

file_dim = data.frame(file_names)    ### convert file name vector to a dataframe so ...
file_dim = dim(file_dim)[1]          ### ... one can determine the dimension of the file vector


output = c()            ### create empty output dataframe that can be used to capture the results of the loop below


### start of loop that extracts the point cloud of the tree and calculates its volume; it does this for every point cloud that is selected above by the user

for(i in 1:file_dim) {
  
  ufc = read.table(file_names[i], sep = ",", header = TRUE) ### read in point cloud from the ATLS
  
  aoi = read.table(file_aoi, sep = " ", header = FALSE)

  colnames(aoi) = c("x", "y", "z", "int")                   ### Define column names
  
  ### Determine the area of interest (aoi) based on the aoi point cloud data
  
  min_x = min(aoi$x)-0.1       ### Determine the minimum x value in the point cloud and further subtract 10 cm to account for slight changes of the aoi dimension e.g., due to snowfall                                         
  
  max_x = max(aoi$x)+0.1       ### Similar to above just for max x
  
  min_y = min(aoi$y)-0.1       ### Similar to above just for max y
  
  max_y = max(aoi$y)+0.1       ### Similar to above just for max y
  
  min_z = min(aoi$z)-0.1       ### Minimum and maximum z values were determined in CloudCompare using the point picking tool 
  
  max_z = max(aoi$z)+0.1
  
  ### Extract the area of interest from the point cloud
  
  extract = which(ufc$x > min_x & ufc$x < max_x)  ### select only x values that fall between the minimum and maximum x value. The values for minimum and maximum x were determined above. Repeat this for y an z.
  
  ufc = ufc[extract,]
  
  extract = which(ufc$y > min_y & ufc$y < max_y)
  
  ufc = ufc[extract,]
  
  extract = which(ufc$z > min_z & ufc$z < max_z)
  
  ufc = ufc[extract,]
  
  ufc = ufc[,1:3]         ### just use x,y, and z column for further analysis (ignore the intensity values)
  
  ufc = as.matrix(ufc)    ### to run the shape3d function below, the ufc dataframe has to be converted into a matrix
  
  plot3d(ufc$x, ufc$y, ufc$z)   ### visual of the 3d point cloud you extracted (this allows you to check if you extracted in fact what you were looking for)
  
 ### compute 3d alpha shape using the using the alphashape3d package function "ashape3d". Set alpha to match your data resolution. 
  shape_3d.025 = ashape3d(ufc, alpha = 0.025, pert = TRUE) 
 ### plot 3d alpha shape
  ashape.plot.025 = plot(shape_3d.025, indexAlpha = 1:1, col = "green") 
 ### compute the volume of the 3d shape
  ashape.volume.025 = volume_ashape3d(shape_3d.025)
  
 ##extract time: change to -11 with dates in 3 digits (e.g. Nov 2017), -10 for 2 digits##
  time = substr(file_names[i], nchar(file_names[i])-10, nchar(file_names[i])-4)
  time = paste(study_year, time, sep="")
  
  volume = cbind(time, ashape.volume.025) 
  
  output = rbind(output, volume) ### add the calculated volume for a given point cloud to the output dataframe you defined before the start of the loop (output = c())
  
}

output = data.frame(output)

output[,1] = as.POSIXct(output[,1], format = "%Y_%j_%H%M") ### Date time conversion using POSIXct
output[,2] = as.numeric(as.character(output[,2]))

### Begin working to add tree weights to table; this experiment had two duplicate left/right trees (code processes one at a time)

tree_weight[,1] = as.POSIXct(tree_weight[,1], format = "%m/%d/%Y %H:%M") ### Date time conversion using POSIXct

colnames(tree_weight) = c("date", "left_tree", "right_tree")

dim_loop = dim(output)[1]-1 #### includes -1 so that last pairing of start/end dates works

weight_output = c()

for( i in 1:dim_loop ){
  
  start_date = output[i,1]
  end_date = output[i+1,1]
  
  extract = which(tree_weight[,1] >= start_date & tree_weight[,1] <= end_date)
  weight.date = tree_weight[extract,]
  mean.weight = mean(weight.date$left_tree) 
  sd.weight = sd(weight.date$left_tree)
  
  out = cbind(mean.weight, sd.weight)
  
  out = data.frame(out)
  
  weight_output = rbind(weight_output, out)
  
}

### Begin working to add air temps to table

temps$date = as.POSIXct(swe[,1], format = "%m/%d/%Y %H:%M")

colnames(temps) = c("date", "air_temp")

dim_loop = dim(output)[1]-1 #### includes -1 so that last pairing of start/end dates works

temps_output = c()

for( i in 1:dim_loop){
  
  start_date = output[i,1]
  end_date = output[i+1,1]
  
  extract = which(temps[,1] >= start_date & temps[,1] <= end_date)
  temps.date = temps[extract,]
  mean.air.temp = mean(temps.date$air_temp) 
  sd.air.temp = sd(temps.date$air_temp)
  
  out = cbind(mean.air.temp, sd.air.temp)
  
  out = data.frame(out)
  
  temps_output = rbind(temps_output, out)
  
}

output = output[1:dim_loop,]

output.volume.weight = cbind(output, weight_output)

output.volume.weight.temps = cbind(output.volume.weight, temps_output)

left.tree.file = write.csv(output.volume.weight.temps, "INSERT CSV LOCATION")

##Manually (in Excel), originating volume of tree (snow-free) was subtracted from each subsequent reading; the originating weight was subtracted from each subsequent reading, then weights were manually converted from mV/V to kg and divided by two to match ATLS.

##Read in manually adjusted table
left.tree.table = read.csv("INSERT CSV LOCATION")

##adding density columns
left.tree.table$density.lowry = 119 + (6.48 * left.tree.table$mean.air.temp)

left.tree.table$density.pomeroy = 67.92 + (51.25*exp(left.tree.table$mean.air.temp/2.59))

left.tree.table$density.lachapelle = 50 + (1.7 * (left.tree.table$mean.air.temp + 15)^1.5)

left.tree.table$density.constant = 100

##check for any NAs and remove those rows

##Figure 2 plotting (left tree)

x_max = 9
x_min = 0

y_max = 9
y_min = 0

par(mfrow = c(2,2)) 
par(pty="s")

plot(half.mean.weight ~ I(ashape.volume.025 * density.constant), ylim = c(y_min, y_max), xlim = c(x_min, x_max), data = left.tree.table, pch = 2, col = "black", ylab = "Load cell mass (kg)", xlab = "TLS mass (kg)", main = "Left tree: ??(constant)[1]")
constant.model.025 = lm(half.mean.weight ~ I(ashape.volume.025 * density.constant), data = left.tree.table)
abline(constant.model.025, lty  = 1, col = "black")
abline(0,1, lty=2, col="red")

plot(half.mean.weight ~ I(ashape.volume.025 * density.lowry), ylim = c(y_min, y_max), xlim = c(x_min, x_max), data = left.tree.table, pch = 2, col = "black", ylab = "Load cell mass (kg)", xlab = "TLS mass (kg)", main = "Left tree: ??(Diamond-Lowry)[2]")
lowry.model.025 = lm(half.mean.weight ~ I(ashape.volume.025 * density.lowry), data = left.tree.table)
abline(lowry.model.025, lty  = 1, col = "black")
abline(0,1, lty=2, col="red")

plot(half.mean.weight ~ I(ashape.volume.025 * density.lachapelle), ylim = c(y_min, y_max), xlim = c(x_min, x_max), data = left.tree.table, pch = 2, col = "black", ylab = "Load cell mass (kg)", xlab = "TLS mass (kg)", main = "Left tree: ??(LaChapelle)[3]")
lachapelle.model.025 = lm(half.mean.weight ~ I(ashape.volume.025 * density.lachapelle), data = left.tree.table)
abline(lachapelle.model.025, lty  = 1, col = "black")
abline(0,1, lty=2, col="red")

plot(half.mean.weight ~ I(ashape.volume.025 * density.pomeroy), ylim = c(y_min, y_max), xlim = c(x_min, x_max), data = left.tree.table, pch = 2, col = "black", ylab = "Load cell mass (kg)", xlab = "TLS mass (kg)", main = "Left tree: ??(Hedstrom-Pomeroy)[4]")
pomeroy.model.025 = lm(half.mean.weight ~ I(ashape.volume.025 * density.pomeroy), data = left.tree.table)
abline(pomeroy.model.025, lty = 1, col = "black")
abline(0,1, lty=2, col="red")

summary(pomeroy.model.025)
summary(constant.model.025)
summary(lowry.model.025)
summary(lachapelle.model.025)






